@@ -37,6 +37,24 @@ class ProbaReason:
         result = self.model.predict_proba(X).max(axis=1) <= self.max_proba
         return result.astype(np.float16)
 
+    @staticmethod
+    def from_proba(proba, max_proba=0.55):
+        """
+        Outputs a reason array from a proba array, skipping the need for a model.
+
+        Usage:
+
+        ```python
+        import numpy as np
+        from doubtlab.reason import ProbaReason
+
+        probas = np.array([[0.9, 0.1], [0.5, 0.5]])
+        predicate = ProbaReason.from_proba(probas)
+        assert np.all(predicate == np.array([0.0, 1.0]))
+        ```
+        """
+        return (proba.max(axis=1) <= max_proba).astype(np.float16)
+
 
 class RandomReason:
     """
@@ -69,7 +87,69 @@ class RandomReason:
     def __call__(self, X, y=None):
         np.random.seed(self.random_seed)
         rvals = np.random.random(size=len(X))
-        return np.where(rvals < self.probability, rvals, 0)
+        return (rvals < self.probability).astype(np.float16)
+
+
+class ShannonEntropyReason:
+    """
+    Assign doubt when the normalized Shannon entropy is too high, see
+    [here](https://math.stackexchange.com/questions/395121/how-entropy-scales-with-sample-size)
+    for a discussion.
+
+    Arguments:
+        model: scikit-learn classifier
+        threshold: confidence threshold for doubt assignment
+        smoothing: constant value added to probas to prevent division by zeor
+
+    Usage:
+
+    ```python
+    from sklearn.datasets import load_iris
+    from sklearn.linear_model import LogisticRegression
+
+    from doubtlab.ensemble import DoubtEnsemble
+    from doubtlab.reason import ShannonEntropyReason
+
+    X, y = load_iris(return_X_y=True)
+    model = LogisticRegression(max_iter=1_000)
+    model.fit(X, y)
+
+    doubt = DoubtEnsemble(reason = ShannonEntropyReason(model=model))
+
+    indices = doubt.get_indices(X, y)
+    ```
+    """
+
+    def __init__(self, model, threshold=0.5, smoothing=1e-5):
+        self.model = model
+        self.threshold = threshold
+        self.smoothing = smoothing
+
+    def __call__(self, X, y):
+        probas = self.model.predict_proba(X)
+        return self.from_proba(
+            probas, threshold=self.threshold, smoothing=self.smoothing
+        )
+
+    @staticmethod
+    def from_proba(proba, threshold=0.5, smoothing=1e-5):
+        """
+        Outputs a reason array from a prediction array, skipping the need for a model.
+
+        Usage:
+
+        ```python
+        import numpy as np
+        from doubtlab.reason import ShannonEntropyReason
+
+        probas = np.array([[0.9, 0.1, 0.0], [0.5, 0.4, 0.1]])
+        predicate = ShannonEntropyReason.from_proba(probas, threshold=0.8)
+        assert np.all(predicate == np.array([0.0, 1.0]))
+        ```
+        """
+        probas = proba + smoothing
+        entropies = -(probas * np.log(probas) / np.log(probas.shape[1])).sum(axis=1)
+        return (entropies > threshold).astype(np.float16)
 
 
 class WrongPredictionReason:
@@ -102,7 +182,27 @@ class WrongPredictionReason:
         self.model = model
 
     def __call__(self, X, y):
-        return (self.model.predict(X) != y).astype(np.float16)
+        preds = self.model.predict(X)
+        return self.from_predict(preds, y)
+
+    @staticmethod
+    def from_predict(pred, y):
+        """
+        Outputs a reason array from a prediction array, skipping the need for a model.
+
+        Usage:
+
+        ```python
+        import numpy as np
+        from doubtlab.reason import WrongPredictionReason
+
+        preds = np.array(["positive", "negative"])
+        y = np.array(["positive", "neutral"])
+        predicate = WrongPredictionReason.from_predict(preds, y)
+        assert np.all(predicate == np.array([0.0, 1.0]))
+        ```
+        """
+        return (pred != y).astype(np.float16)
 
 
 class LongConfidenceReason:
@@ -136,24 +236,40 @@ class LongConfidenceReason:
         self.model = model
         self.threshold = threshold
 
-    def _max_bad_class_confidence(self, X, y):
-        probas = self.model.predict_proba(X)
+    @staticmethod
+    def from_proba(proba, y, classes, threshold):
+        """
+        Outputs a reason array from a proba array, skipping the need for a model.
+
+        Usage:
+
+        ```python
+        import numpy as np
+        from doubtlab.reason import LongConfidenceReason
+
+        probas = np.array([[0.9, 0.1], [0.5, 0.5], [0.2, 0.8]])
+        y = np.array([0, 1, 0])
+        classes = np.array([0, 1])
+        threshold = 0.4
+        predicate = LongConfidenceReason.from_proba(probas, y, classes, threshold)
+        assert np.all(predicate == np.array([0.0, 1.0, 1.0]))
+        ```
+        """
         values = []
-        for i, proba in enumerate(probas):
-            proba_dict = {
-                self.model.classes_[j]: v for j, v in enumerate(proba) if j != y[i]
-            }
+        for i, proba in enumerate(proba):
+            proba_dict = {classes[j]: v for j, v in enumerate(proba) if j != y[i]}
             values.append(max(proba_dict.values()))
-        return np.array(values)
+        confidences = np.array(values)
+        return (confidences > threshold).astype(np.float16)
 
     def __call__(self, X, y):
-        confidences = self._max_bad_class_confidence(X, y)
-        return np.where(confidences > self.threshold, confidences, 0)
+        probas = self.model.predict_proba(X)
+        return self.from_proba(probas, y, self.model.classes_, self.threshold)
 
 
 class MarginConfidenceReason:
     """
-    Assign doubt when a the difference between the top two most confident classes is too large.
+    Assign doubt when the difference between the top two most confident classes is too small.
 
     Throws an error when there are only two classes.
 
@@ -184,14 +300,29 @@ class MarginConfidenceReason:
         self.model = model
         self.threshold = threshold
 
-    def _calc_margin(self, probas):
-        sorted = np.sort(probas, axis=1)
-        return sorted[:, -1] - sorted[:, -2]
+    @staticmethod
+    def from_proba(proba, threshold=0.2):
+        """
+        Outputs a reason array from a proba array, skipping the need for a model.
+
+        Usage:
+
+        ```python
+        import numpy as np
+        from doubtlab.reason import MarginConfidenceReason
+
+        probas = np.array([[0.9, 0.1, 0.0], [0.5, 0.4, 0.1]])
+        predicate = MarginConfidenceReason.from_proba(probas, threshold=0.3)
+        assert np.all(predicate == np.array([0.0, 1.0]))
+        ```
+        """
+        sorted = np.sort(proba, axis=1)
+        margin = sorted[:, -1] - sorted[:, -2]
+        return (margin < threshold).astype(np.float16)
 
     def __call__(self, X, y):
         probas = self.model.predict_proba(X)
-        margin = self._calc_margin(probas)
-        return np.where(margin > self.threshold, margin, 0)
+        return self.from_proba(probas, self.threshold)
 
 
 class ShortConfidenceReason:
@@ -215,7 +346,7 @@ class ShortConfidenceReason:
     model = LogisticRegression(max_iter=1_000)
     model.fit(X, y)
 
-    doubt = DoubtEnsemble(reason = ShortConfidenceReason(model=model))
+    doubt = DoubtEnsemble(reason = ShortConfidenceReason(model=model, threshold=0.4))
 
     indices = doubt.get_indices(X, y)
     ```
@@ -225,21 +356,35 @@ class ShortConfidenceReason:
         self.model = model
         self.threshold = threshold
 
-    def _correct_class_confidence(self, X, y):
+    @staticmethod
+    def from_proba(proba, y, classes, threshold=0.2):
         """
-        Gives the predicted confidence (or proba) associated
-        with the correct label `y` from a given model.
+        Outputs a reason array from a proba array, skipping the need for a model.
+
+        Usage:
+
+        ```python
+        import numpy as np
+        from doubtlab.reason import ShortConfidenceReason
+
+        probas = np.array([[0.9, 0.1], [0.5, 0.5], [0.3, 0.7]])
+        y = np.array([0, 1, 0])
+        classes = np.array([0, 1])
+        threshold = 0.4
+        predicate = ShortConfidenceReason.from_proba(probas, y, classes, threshold)
+        assert np.all(predicate == np.array([0.0, 0.0, 1.0]))
+        ```
         """
-        probas = self.model.predict_proba(X)
         values = []
-        for i, proba in enumerate(probas):
-            proba_dict = {self.model.classes_[j]: v for j, v in enumerate(proba)}
+        for i, p in enumerate(proba):
+            proba_dict = {classes[j]: v for j, v in enumerate(p)}
             values.append(proba_dict[y[i]])
-        return np.array(values)
+        confidences = np.array(values)
+        return (confidences < threshold).astype(np.float16)
 
     def __call__(self, X, y):
-        confidences = self._correct_class_confidence(X, y)
-        return np.where(confidences < self.threshold, 1 - confidences, 0)
+        probas = self.model.predict_proba(X)
+        return self.from_proba(probas, y, self.model.classes_, self.threshold)
 
 
 class DisagreeReason:
@@ -276,9 +421,29 @@ class DisagreeReason:
         self.model1 = model1
         self.model2 = model2
 
+    @staticmethod
+    def from_pred(pred1, pred2):
+        """
+        Outputs a reason array from two pred arrays, skipping the need for a model.
+
+        Usage:
+
+        ```python
+        import numpy as np
+        from doubtlab.reason import DisagreeReason
+
+        pred1 = [0, 1, 2]
+        pred2 = [0, 1, 1]
+        predicate = DisagreeReason.from_pred(pred1, pred2)
+        assert np.all(predicate == np.array([0.0, 0.0, 1.0]))
+        ```
+        """
+        return (np.array(pred1) != np.array(pred2)).astype(np.float16)
+
     def __call__(self, X, y):
-        result = self.model1.predict(X) != self.model2.predict(X)
-        return result.astype(np.float16)
+        pred1 = self.model1.predict(X)
+        pred2 = self.model2.predict(X)
+        return self.from_pred(pred1, pred2)
 
 
 class OutlierReason:
@@ -420,11 +585,29 @@ class CleanlabReason:
         self.sorted_index_method = sorted_index_method
         self.min_doubt = min_doubt
 
+    @staticmethod
+    def from_proba(proba, y, min_doubt=0.5, sorted_index_method="normalized_margin"):
+        """
+        Outputs a reason array from a proba array, skipping the need for a model.
+
+        Usage:
+
+        ```python
+        import numpy as np
+        from doubtlab.reason import CleanlabReason
+
+        probas = np.array([[0.9, 0.1], [0.5, 0.5]])
+        y = np.array([0, 1])
+        predicate = CleanlabReason.from_proba(probas, y)
+        ```
+        """
+        ordered_label_errors = get_noise_indices(y, proba, sorted_index_method)
+        result = np.zeros_like(y)
+        conf_arr = np.linspace(1, min_doubt, result.shape[0])
+        for idx, _ in zip(ordered_label_errors, conf_arr):
+            result[idx] = 1
+        return result.astype(np.float16)
+
     def __call__(self, X, y):
         probas = self.model.predict_proba(X)
-        ordered_label_errors = get_noise_indices(y, probas, self.sorted_index_method)
-        result = np.zeros_like(y)
-        conf_arr = np.linspace(1, self.min_doubt, result.shape[0])
-        for idx, conf in zip(ordered_label_errors, conf_arr):
-            result[idx] = conf
-        return result
+        return self.from_proba(probas, y, self.min_doubt, self.sorted_index_method)
